@@ -28,10 +28,18 @@ from slixmpp.exceptions import IqError, IqTimeout
 from slixmpp_omemo import PluginCouldNotLoad, MissingOwnKey, NoAvailableSession
 from slixmpp_omemo import UndecidedException, UntrustedException, EncryptionPrepareException
 import slixmpp_omemo
+from pathlib import Path
 
 log = logging.getLogger(__name__)
 
 
+def jid_as_path(jid: JID) -> Path:
+    """Ensure JID in folder names don't contain illegal chars for the FS"""
+    jid_str = jid.bare.encode('utf-8')
+    digest = hashlib.sha256(jid_str).digest()
+    return Path(base64.b32encode(digest).decode('US-ASCII'))
+
+
 class Plugin(E2EEPlugin):
     """OMEMO (XEP-0384) Plugin"""
 
@@ -70,11 +78,17 @@ class Plugin(E2EEPlugin):
 
         self.info = lambda i: self.api.information(i, 'Info')
 
-        jid_str = self.core.xmpp.boundjid.bare.encode('utf-8')
-        digest = hashlib.sha256(jid_str).digest()
-        hashed_jid = base64.b32encode(digest).decode('US-ASCII')
-        data_dir = os.path.join(DATA_HOME, 'omemo', hashed_jid)
-        os.makedirs(data_dir, exist_ok=True)
+        data_dir = os.path.join(
+            DATA_HOME,
+            'omemo',
+            jid_as_path(self.core.xmpp.boundjid),
+        )
+
+        try:
+            # Raise exception if folder exists so that we don't chmod again.
+            os.makedirs(data_dir, mode=0o700, exist_ok=False)
+        except OSError:  # Folder already exists
+            pass
 
         try:
             self.core.xmpp.register_plugin(
@@ -109,14 +123,7 @@ class Plugin(E2EEPlugin):
             if trust is not None
         ]
 
-    def get_omemo_sid_message(self) -> str:
-        return 'xmpp:{bare}?omemo-sid-{did}={fp}'.format(
-            bare=self.core.xmpp.boundjid.bare,
-            did=self.my_device_id(),
-            key=slixmpp_omemo.fp_from_ik(self.my_identity_key()),
-        )
-
-    def decrypt(self, message: Message, jid: Optional[JID], tab: ChatTab) -> None:
+    async def decrypt(self, message: Message, jid: Optional[JID], tab: ChatTab) -> None:
         if jid is None:
             self.display_error('Unable to decrypt the message.')
             return None
@@ -133,13 +140,15 @@ class Plugin(E2EEPlugin):
         body = None
         try:
             encrypted = message['omemo_encrypted']
-            body = self.core.xmpp['xep_0384'].decrypt_message(
+            body = await self.core.xmpp['xep_0384'].decrypt_message(
                 encrypted,
                 jid,
                 # Always decrypt. Let us handle how we then warn the user.
                 allow_untrusted=True,
             )
-            body = body.decode('utf-8')
+            # Heartbeats will return a None body.
+            if body is not None:
+                body = body.decode('utf-8')
         except (MissingOwnKey,):
             # The message is missing our own key, it was not encrypted for
             # us, and we can't decrypt it.
@@ -151,7 +160,7 @@ class Plugin(E2EEPlugin):
                     'I can\'t decrypt this message as it '
                     'is not encrypted for me.'
                 )
-        except (NoAvailableSession,) as exn:
+        except (NoAvailableSession,):
             # We received a message from that contained a session that we
             # don't know about (deleted session storage, etc.). We can't
             # decrypt the message, and it's going to be lost.
@@ -168,7 +177,7 @@ class Plugin(E2EEPlugin):
             # and given a chance to resolve them already.
             self.display_error('I was not able to decrypt the message.')
         except (Exception,) as exn:
-            self.display_error('An error occured while attempting decryption.\n%r' % exn)
+            self.display_error(f'An error occured while attempting decryption.\n{exn}')
             raise
 
         if body is not None:
@@ -207,23 +216,28 @@ class Plugin(E2EEPlugin):
                 # untrusted/undecided barejid, so we need to make a decision here.
                 # This is where you prompt your user to ask what to do. In
                 # this bot we will automatically trust undecided recipients.
-                self.core.xmpp['xep_0384'].trust(exn.bare_jid, exn.device, exn.ik)
+                await self.core.xmpp['xep_0384'].trust(exn.bare_jid, exn.device, exn.ik)
             # TODO: catch NoEligibleDevicesException
             except EncryptionPrepareException as exn:
                 log.debug('FOO: EncryptionPrepareException: %r', exn.errors)
                 for error in exn.errors:
                     if isinstance(error, MissingBundleException):
                         self.display_error(
-                            'Could not find keys for device "%d" of recipient "%s". Skipping.' %
-                            (error.device, error.bare_jid),
+                            f'Could not find keys for device "{error.device}" '
+                            f'of recipient "{error.bare_jid}". Skipping.'
                         )
                         jid = JID(error.bare_jid)
                         device_list = expect_problems.setdefault(jid, [])
                         device_list.append(error.device)
             except (IqError, IqTimeout) as exn:
                 self.display_error(
-                    'An error occured while fetching information on a recipient.\n%r' % exn,
+                    'An error occured while fetching information on a recipient.\n{exn}'
                 )
                 return None
 
         return None
+
+
+if __name__ == '__main__':
+    jid = JID(input('JID: '))
+    print(jid_as_path(jid))
